library("ggplot2")
library("reshape2")
library("plyr")

data <- read.table("results.dat", header=TRUE)
data <- data[data$separation < 80,]
#data <- data[data$threads == 4,]
#data <- data[data$separation % 8 ==0,]

mdata <- melt(data, id.vars=c("threads", "separation"))
cdata <- ddply(mdata, .(threads, separation), summarise, min=min(value),
               max=max(value), avg=mean(value), med=quantile(value, 0.5),
               low=quantile(value, 0.25), high=quantile(value, 0.75))


ggplot(data=cdata, aes(x=separation, y=med, color=as.factor(threads),
                      ymin=low, ymax=high)) +
  geom_point(alpha=1) +
  geom_errorbar(alpha=1) +
  scale_x_continuous("Stride [Byte]") +
  scale_y_continuous("Cummulative Time [s]") +
  scale_color_discrete("Threads") +
  theme_bw(30) +
  theme(legend.position=c(0.8, 0.7)) 

                